/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.3.0 (build: 2013-05-14 03:07:32 UTC)
 *  on 2013-05-16 at 16:30:37 UTC 
 */

package com.google.api.services.admin.directory;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Directory (directory_v1).
 *
 * <p>
 * Apps Directory API lets you view and manage enterprise resources like user, groups, OrgUnit, devices.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/google-apps/cloud-directory/v1/concepts.html" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link DirectoryRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Directory extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Google Apps Directory API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "admin/directory/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Directory(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Directory(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Chromeosdevices collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Chromeosdevices.List request = admin.chromeosdevices().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Chromeosdevices chromeosdevices() {
    return new Chromeosdevices();
  }

  /**
   * The "chromeosdevices" collection of methods.
   */
  public class Chromeosdevices {

    /**
     * Retrieve Chrome OS Device
     *
     * Create a request for the method "chromeosdevices.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param deviceId Immutable id of Chrome OS Device
     * @return the request
     */
    public Get get(String customerId, String deviceId) throws java.io.IOException {
      Get result = new Get(customerId, deviceId);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos/{deviceId}";

      /**
       * Retrieve Chrome OS Device
       *
       * Create a request for the method "chromeosdevices.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param deviceId Immutable id of Chrome OS Device
       * @since 1.13
       */
      protected Get(String customerId, String deviceId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.ChromeOsDevice.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.deviceId = Preconditions.checkNotNull(deviceId, "Required parameter deviceId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Get setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Chrome OS Device */
      @com.google.api.client.util.Key
      private String deviceId;

      /** Immutable id of Chrome OS Device
       */
      public String getDeviceId() {
        return deviceId;
      }

      /** Immutable id of Chrome OS Device */
      public Get setDeviceId(String deviceId) {
        this.deviceId = deviceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Get setProjection(String projection) {
        this.projection = projection;
        return this;
      }

    }
    /**
     * Retrieve all Chrome OS Devices of a customer (paginated)
     *
     * Create a request for the method "chromeosdevices.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @return the request
     */
    public List list(String customerId) throws java.io.IOException {
      List result = new List(customerId);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevices> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos";

      /**
       * Retrieve all Chrome OS Devices of a customer (paginated)
       *
       * Create a request for the method "chromeosdevices.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @since 1.13
       */
      protected List(String customerId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.ChromeOsDevices.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public List setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Column to use for sorting results */
      @com.google.api.client.util.Key
      private String orderBy;

      /** Column to use for sorting results
       */
      public String getOrderBy() {
        return orderBy;
      }

      /** Column to use for sorting results */
      public List setOrderBy(String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public List setProjection(String projection) {
        this.projection = projection;
        return this;
      }

      /** Maximum number of results to return. Default is 100 */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Maximum number of results to return. Default is 100

     [minimum: 1]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 100 */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page in the list
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Whether to return results in ascending or descending order. Only of use when orderBy is also
     * used
     */
      @com.google.api.client.util.Key
      private String sortOrder;

      /** Whether to return results in ascending or descending order. Only of use when orderBy is also used
       */
      public String getSortOrder() {
        return sortOrder;
      }

      /**
     * Whether to return results in ascending or descending order. Only of use when orderBy is also
     * used
     */
      public List setSortOrder(String sortOrder) {
        this.sortOrder = sortOrder;
        return this;
      }

      /**
     * Search string in the format given at
     * http://support.google.com/chromeos/a/bin/answer.py?hl=en=1698333
     */
      @com.google.api.client.util.Key
      private String query;

      /** Search string in the format given at
     http://support.google.com/chromeos/a/bin/answer.py?hl=en=1698333
       */
      public String getQuery() {
        return query;
      }

      /**
     * Search string in the format given at
     * http://support.google.com/chromeos/a/bin/answer.py?hl=en=1698333
     */
      public List setQuery(String query) {
        this.query = query;
        return this;
      }

    }
    /**
     * Update Chrome OS Device. This method supports patch semantics.
     *
     * Create a request for the method "chromeosdevices.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param deviceId Immutable id of Chrome OS Device
     * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
     * @return the request
     */
    public Patch patch(String customerId, String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) throws java.io.IOException {
      Patch result = new Patch(customerId, deviceId, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos/{deviceId}";

      /**
       * Update Chrome OS Device. This method supports patch semantics.
       *
       * Create a request for the method "chromeosdevices.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param deviceId Immutable id of Chrome OS Device
       * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
       * @since 1.13
       */
      protected Patch(String customerId, String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.ChromeOsDevice.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.deviceId = Preconditions.checkNotNull(deviceId, "Required parameter deviceId must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Patch setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Chrome OS Device */
      @com.google.api.client.util.Key
      private String deviceId;

      /** Immutable id of Chrome OS Device
       */
      public String getDeviceId() {
        return deviceId;
      }

      /** Immutable id of Chrome OS Device */
      public Patch setDeviceId(String deviceId) {
        this.deviceId = deviceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Patch setProjection(String projection) {
        this.projection = projection;
        return this;
      }

    }
    /**
     * Update Chrome OS Device
     *
     * Create a request for the method "chromeosdevices.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param deviceId Immutable id of Chrome OS Device
     * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
     * @return the request
     */
    public Update update(String customerId, String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) throws java.io.IOException {
      Update result = new Update(customerId, deviceId, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos/{deviceId}";

      /**
       * Update Chrome OS Device
       *
       * Create a request for the method "chromeosdevices.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param deviceId Immutable id of Chrome OS Device
       * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
       * @since 1.13
       */
      protected Update(String customerId, String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.ChromeOsDevice.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.deviceId = Preconditions.checkNotNull(deviceId, "Required parameter deviceId must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Update setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Chrome OS Device */
      @com.google.api.client.util.Key
      private String deviceId;

      /** Immutable id of Chrome OS Device
       */
      public String getDeviceId() {
        return deviceId;
      }

      /** Immutable id of Chrome OS Device */
      public Update setDeviceId(String deviceId) {
        this.deviceId = deviceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Update setProjection(String projection) {
        this.projection = projection;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Groups collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Groups.List request = admin.groups().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Groups groups() {
    return new Groups();
  }

  /**
   * The "groups" collection of methods.
   */
  public class Groups {

    /**
     * Delete Group
     *
     * Create a request for the method "groups.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @return the request
     */
    public Delete delete(String groupKey) throws java.io.IOException {
      Delete result = new Delete(groupKey);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Delete Group
       *
       * Create a request for the method "groups.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @since 1.13
       */
      protected Delete(String groupKey) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Delete setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

    }
    /**
     * Retrieve Group
     *
     * Create a request for the method "groups.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @return the request
     */
    public Get get(String groupKey) throws java.io.IOException {
      Get result = new Get(groupKey);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Retrieve Group
       *
       * Create a request for the method "groups.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @since 1.13
       */
      protected Get(String groupKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Group.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Get setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

    }
    /**
     * Create Group
     *
     * Create a request for the method "groups.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.admin.directory.model.Group}
     * @return the request
     */
    public Insert insert(com.google.api.services.admin.directory.model.Group content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups";

      /**
       * Create Group
       *
       * Create a request for the method "groups.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.admin.directory.model.Group}
       * @since 1.13
       */
      protected Insert(com.google.api.services.admin.directory.model.Group content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Group.class);
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

    }
    /**
     * Retrieve all groups in a domain (paginated)
     *
     * Create a request for the method "groups.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Groups> {

      private static final String REST_PATH = "groups";

      /**
       * Retrieve all groups in a domain (paginated)
       *
       * Create a request for the method "groups.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Groups.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * Immutable id of the Google Apps account. In case of multi-domain, to fetch all groups for a
     * customer, fill this field instead of domain.
     */
      @com.google.api.client.util.Key
      private String customer;

      /** Immutable id of the Google Apps account. In case of multi-domain, to fetch all groups for a
     customer, fill this field instead of domain.
       */
      public String getCustomer() {
        return customer;
      }

      /**
     * Immutable id of the Google Apps account. In case of multi-domain, to fetch all groups for a
     * customer, fill this field instead of domain.
     */
      public List setCustomer(String customer) {
        this.customer = customer;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page in the list
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Name of the domain. Fill this field to get groups from only this domain. To return all groups
     * in a multi-domain fill customer field instead.
     */
      @com.google.api.client.util.Key
      private String domain;

      /** Name of the domain. Fill this field to get groups from only this domain. To return all groups in a
     multi-domain fill customer field instead.
       */
      public String getDomain() {
        return domain;
      }

      /**
     * Name of the domain. Fill this field to get groups from only this domain. To return all groups
     * in a multi-domain fill customer field instead.
     */
      public List setDomain(String domain) {
        this.domain = domain;
        return this;
      }

      /** Maximum number of results to return. Default is 200 */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Maximum number of results to return. Default is 200

     [minimum: 1]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 200 */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /**
     * Email or immutable Id of the user if only those groups are to be listed, the given user is a
     * member of. If Id, it should match with id of user object
     */
      @com.google.api.client.util.Key
      private String userKey;

      /** Email or immutable Id of the user if only those groups are to be listed, the given user is a member
     of. If Id, it should match with id of user object
       */
      public String getUserKey() {
        return userKey;
      }

      /**
     * Email or immutable Id of the user if only those groups are to be listed, the given user is a
     * member of. If Id, it should match with id of user object
     */
      public List setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }
    /**
     * Update Group. This method supports patch semantics.
     *
     * Create a request for the method "groups.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param content the {@link com.google.api.services.admin.directory.model.Group}
     * @return the request
     */
    public Patch patch(String groupKey, com.google.api.services.admin.directory.model.Group content) throws java.io.IOException {
      Patch result = new Patch(groupKey, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Update Group. This method supports patch semantics.
       *
       * Create a request for the method "groups.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param content the {@link com.google.api.services.admin.directory.model.Group}
       * @since 1.13
       */
      protected Patch(String groupKey, com.google.api.services.admin.directory.model.Group content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.Group.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Patch setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

    }
    /**
     * Update Group
     *
     * Create a request for the method "groups.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param content the {@link com.google.api.services.admin.directory.model.Group}
     * @return the request
     */
    public Update update(String groupKey, com.google.api.services.admin.directory.model.Group content) throws java.io.IOException {
      Update result = new Update(groupKey, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Update Group
       *
       * Create a request for the method "groups.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param content the {@link com.google.api.services.admin.directory.model.Group}
       * @since 1.13
       */
      protected Update(String groupKey, com.google.api.services.admin.directory.model.Group content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.Group.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Update setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

    }

    /**
     * An accessor for creating requests from the Aliases collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Directory admin = new Directory(...);}
     *   {@code Directory.Aliases.List request = admin.aliases().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Aliases aliases() {
      return new Aliases();
    }

    /**
     * The "aliases" collection of methods.
     */
    public class Aliases {

      /**
       * Remove a alias for the group
       *
       * Create a request for the method "aliases.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param groupKey Email or immutable Id of the group
       * @param alias The alias to be removed
       * @return the request
       */
      public Delete delete(String groupKey, String alias) throws java.io.IOException {
        Delete result = new Delete(groupKey, alias);
        initialize(result);
        return result;
      }

      public class Delete extends DirectoryRequest<Void> {

        private static final String REST_PATH = "groups/{groupKey}/aliases/{alias}";

        /**
         * Remove a alias for the group
         *
         * Create a request for the method "aliases.delete".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param groupKey Email or immutable Id of the group
         * @param alias The alias to be removed
         * @since 1.13
         */
        protected Delete(String groupKey, String alias) {
          super(Directory.this, "DELETE", REST_PATH, null, Void.class);
          this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
          this.alias = Preconditions.checkNotNull(alias, "Required parameter alias must be specified.");
        }

        @Override
        public Delete setAlt(String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the group */
        @com.google.api.client.util.Key
        private String groupKey;

        /** Email or immutable Id of the group
         */
        public String getGroupKey() {
          return groupKey;
        }

        /** Email or immutable Id of the group */
        public Delete setGroupKey(String groupKey) {
          this.groupKey = groupKey;
          return this;
        }

        /** The alias to be removed */
        @com.google.api.client.util.Key
        private String alias;

        /** The alias to be removed
         */
        public String getAlias() {
          return alias;
        }

        /** The alias to be removed */
        public Delete setAlias(String alias) {
          this.alias = alias;
          return this;
        }

      }
      /**
       * Add a alias for the group
       *
       * Create a request for the method "aliases.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       *
       * @param groupKey Email or immutable Id of the group
       * @param content the {@link com.google.api.services.admin.directory.model.Alias}
       * @return the request
       */
      public Insert insert(String groupKey, com.google.api.services.admin.directory.model.Alias content) throws java.io.IOException {
        Insert result = new Insert(groupKey, content);
        initialize(result);
        return result;
      }

      public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Alias> {

        private static final String REST_PATH = "groups/{groupKey}/aliases";

        /**
         * Add a alias for the group
         *
         * Create a request for the method "aliases.insert".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
         * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param groupKey Email or immutable Id of the group
         * @param content the {@link com.google.api.services.admin.directory.model.Alias}
         * @since 1.13
         */
        protected Insert(String groupKey, com.google.api.services.admin.directory.model.Alias content) {
          super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Alias.class);
          this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        }

        @Override
        public Insert setAlt(String alt) {
          return (Insert) super.setAlt(alt);
        }

        @Override
        public Insert setFields(String fields) {
          return (Insert) super.setFields(fields);
        }

        @Override
        public Insert setKey(String key) {
          return (Insert) super.setKey(key);
        }

        @Override
        public Insert setOauthToken(String oauthToken) {
          return (Insert) super.setOauthToken(oauthToken);
        }

        @Override
        public Insert setPrettyPrint(Boolean prettyPrint) {
          return (Insert) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Insert setQuotaUser(String quotaUser) {
          return (Insert) super.setQuotaUser(quotaUser);
        }

        @Override
        public Insert setUserIp(String userIp) {
          return (Insert) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the group */
        @com.google.api.client.util.Key
        private String groupKey;

        /** Email or immutable Id of the group
         */
        public String getGroupKey() {
          return groupKey;
        }

        /** Email or immutable Id of the group */
        public Insert setGroupKey(String groupKey) {
          this.groupKey = groupKey;
          return this;
        }

      }
      /**
       * List all aliases for a group
       *
       * Create a request for the method "aliases.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation.
       *
       * @param groupKey Email or immutable Id of the group
       * @return the request
       */
      public List list(String groupKey) throws java.io.IOException {
        List result = new List(groupKey);
        initialize(result);
        return result;
      }

      public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Aliases> {

        private static final String REST_PATH = "groups/{groupKey}/aliases";

        /**
         * List all aliases for a group
         *
         * Create a request for the method "aliases.list".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
         * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param groupKey Email or immutable Id of the group
         * @since 1.13
         */
        protected List(String groupKey) {
          super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Aliases.class);
          this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public List setAlt(String alt) {
          return (List) super.setAlt(alt);
        }

        @Override
        public List setFields(String fields) {
          return (List) super.setFields(fields);
        }

        @Override
        public List setKey(String key) {
          return (List) super.setKey(key);
        }

        @Override
        public List setOauthToken(String oauthToken) {
          return (List) super.setOauthToken(oauthToken);
        }

        @Override
        public List setPrettyPrint(Boolean prettyPrint) {
          return (List) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public List setQuotaUser(String quotaUser) {
          return (List) super.setQuotaUser(quotaUser);
        }

        @Override
        public List setUserIp(String userIp) {
          return (List) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the group */
        @com.google.api.client.util.Key
        private String groupKey;

        /** Email or immutable Id of the group
         */
        public String getGroupKey() {
          return groupKey;
        }

        /** Email or immutable Id of the group */
        public List setGroupKey(String groupKey) {
          this.groupKey = groupKey;
          return this;
        }

      }

    }
  }

  /**
   * An accessor for creating requests from the Members collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Members.List request = admin.members().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Members members() {
    return new Members();
  }

  /**
   * The "members" collection of methods.
   */
  public class Members {

    /**
     * Remove membership.
     *
     * Create a request for the method "members.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @param memberKey Email or immutable Id of the member
     * @return the request
     */
    public Delete delete(String groupKey, String memberKey) throws java.io.IOException {
      Delete result = new Delete(groupKey, memberKey);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Remove membership.
       *
       * Create a request for the method "members.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @param memberKey Email or immutable Id of the member
       * @since 1.13
       */
      protected Delete(String groupKey, String memberKey) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Delete setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the member */
      @com.google.api.client.util.Key
      private String memberKey;

      /** Email or immutable Id of the member
       */
      public String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the member */
      public Delete setMemberKey(String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

    }
    /**
     * Retrieve Group Member
     *
     * Create a request for the method "members.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @param memberKey Email or immutable Id of the member
     * @return the request
     */
    public Get get(String groupKey, String memberKey) throws java.io.IOException {
      Get result = new Get(groupKey, memberKey);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Retrieve Group Member
       *
       * Create a request for the method "members.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @param memberKey Email or immutable Id of the member
       * @since 1.13
       */
      protected Get(String groupKey, String memberKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Get setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the member */
      @com.google.api.client.util.Key
      private String memberKey;

      /** Email or immutable Id of the member
       */
      public String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the member */
      public Get setMemberKey(String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

    }
    /**
     * Add user to the specified group.
     *
     * Create a request for the method "members.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @param content the {@link com.google.api.services.admin.directory.model.Member}
     * @return the request
     */
    public Insert insert(String groupKey, com.google.api.services.admin.directory.model.Member content) throws java.io.IOException {
      Insert result = new Insert(groupKey, content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members";

      /**
       * Add user to the specified group.
       *
       * Create a request for the method "members.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @param content the {@link com.google.api.services.admin.directory.model.Member}
       * @since 1.13
       */
      protected Insert(String groupKey, com.google.api.services.admin.directory.model.Member content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Insert setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

    }
    /**
     * Retrieve all members in a group (paginated)
     *
     * Create a request for the method "members.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @return the request
     */
    public List list(String groupKey) throws java.io.IOException {
      List result = new List(groupKey);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Members> {

      private static final String REST_PATH = "groups/{groupKey}/members";

      /**
       * Retrieve all members in a group (paginated)
       *
       * Create a request for the method "members.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @since 1.13
       */
      protected List(String groupKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Members.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public List setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page in the list
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Comma separated role values to filter list results on. */
      @com.google.api.client.util.Key
      private String roles;

      /** Comma separated role values to filter list results on.
       */
      public String getRoles() {
        return roles;
      }

      /** Comma separated role values to filter list results on. */
      public List setRoles(String roles) {
        this.roles = roles;
        return this;
      }

      /** Maximum number of results to return. Default is 200 */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Maximum number of results to return. Default is 200

     [minimum: 1]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 200 */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

    }
    /**
     * Update membership of a user in the specified group. This method supports patch semantics.
     *
     * Create a request for the method "members.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
     * @param content the {@link com.google.api.services.admin.directory.model.Member}
     * @return the request
     */
    public Patch patch(String groupKey, String memberKey, com.google.api.services.admin.directory.model.Member content) throws java.io.IOException {
      Patch result = new Patch(groupKey, memberKey, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Update membership of a user in the specified group. This method supports patch semantics.
       *
       * Create a request for the method "members.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
       * @param content the {@link com.google.api.services.admin.directory.model.Member}
       * @since 1.13
       */
      protected Patch(String groupKey, String memberKey, com.google.api.services.admin.directory.model.Member content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Patch setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      @com.google.api.client.util.Key
      private String memberKey;

      /** Email or immutable Id of the user. If Id, it should match with id of member object
       */
      public String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      public Patch setMemberKey(String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

    }
    /**
     * Update membership of a user in the specified group.
     *
     * Create a request for the method "members.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
     * @param content the {@link com.google.api.services.admin.directory.model.Member}
     * @return the request
     */
    public Update update(String groupKey, String memberKey, com.google.api.services.admin.directory.model.Member content) throws java.io.IOException {
      Update result = new Update(groupKey, memberKey, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Update membership of a user in the specified group.
       *
       * Create a request for the method "members.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
       * @param content the {@link com.google.api.services.admin.directory.model.Member}
       * @since 1.13
       */
      protected Update(String groupKey, String memberKey, com.google.api.services.admin.directory.model.Member content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Update setGroupKey(String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      @com.google.api.client.util.Key
      private String memberKey;

      /** Email or immutable Id of the user. If Id, it should match with id of member object
       */
      public String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      public Update setMemberKey(String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Mobiledevices collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Mobiledevices.List request = admin.mobiledevices().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Mobiledevices mobiledevices() {
    return new Mobiledevices();
  }

  /**
   * The "mobiledevices" collection of methods.
   */
  public class Mobiledevices {

    /**
     * Take action on Mobile Device
     *
     * Create a request for the method "mobiledevices.action".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Action#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param resourceId Immutable id of Mobile Device
     * @param content the {@link com.google.api.services.admin.directory.model.MobileDeviceAction}
     * @return the request
     */
    public Action action(String customerId, String resourceId, com.google.api.services.admin.directory.model.MobileDeviceAction content) throws java.io.IOException {
      Action result = new Action(customerId, resourceId, content);
      initialize(result);
      return result;
    }

    public class Action extends DirectoryRequest<Void> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile/{resourceId}/action";

      /**
       * Take action on Mobile Device
       *
       * Create a request for the method "mobiledevices.action".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Action#execute()} method to invoke the remote operation. <p> {@link
       * Action#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param resourceId Immutable id of Mobile Device
       * @param content the {@link com.google.api.services.admin.directory.model.MobileDeviceAction}
       * @since 1.13
       */
      protected Action(String customerId, String resourceId, com.google.api.services.admin.directory.model.MobileDeviceAction content) {
        super(Directory.this, "POST", REST_PATH, content, Void.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.resourceId = Preconditions.checkNotNull(resourceId, "Required parameter resourceId must be specified.");
      }

      @Override
      public Action setAlt(String alt) {
        return (Action) super.setAlt(alt);
      }

      @Override
      public Action setFields(String fields) {
        return (Action) super.setFields(fields);
      }

      @Override
      public Action setKey(String key) {
        return (Action) super.setKey(key);
      }

      @Override
      public Action setOauthToken(String oauthToken) {
        return (Action) super.setOauthToken(oauthToken);
      }

      @Override
      public Action setPrettyPrint(Boolean prettyPrint) {
        return (Action) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Action setQuotaUser(String quotaUser) {
        return (Action) super.setQuotaUser(quotaUser);
      }

      @Override
      public Action setUserIp(String userIp) {
        return (Action) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Action setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Mobile Device */
      @com.google.api.client.util.Key
      private String resourceId;

      /** Immutable id of Mobile Device
       */
      public String getResourceId() {
        return resourceId;
      }

      /** Immutable id of Mobile Device */
      public Action setResourceId(String resourceId) {
        this.resourceId = resourceId;
        return this;
      }

    }
    /**
     * Delete Mobile Device
     *
     * Create a request for the method "mobiledevices.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param resourceId Immutable id of Mobile Device
     * @return the request
     */
    public Delete delete(String customerId, String resourceId) throws java.io.IOException {
      Delete result = new Delete(customerId, resourceId);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile/{resourceId}";

      /**
       * Delete Mobile Device
       *
       * Create a request for the method "mobiledevices.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param resourceId Immutable id of Mobile Device
       * @since 1.13
       */
      protected Delete(String customerId, String resourceId) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.resourceId = Preconditions.checkNotNull(resourceId, "Required parameter resourceId must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Delete setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Mobile Device */
      @com.google.api.client.util.Key
      private String resourceId;

      /** Immutable id of Mobile Device
       */
      public String getResourceId() {
        return resourceId;
      }

      /** Immutable id of Mobile Device */
      public Delete setResourceId(String resourceId) {
        this.resourceId = resourceId;
        return this;
      }

    }
    /**
     * Retrieve Mobile Device
     *
     * Create a request for the method "mobiledevices.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param resourceId Immutable id of Mobile Device
     * @return the request
     */
    public Get get(String customerId, String resourceId) throws java.io.IOException {
      Get result = new Get(customerId, resourceId);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.MobileDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile/{resourceId}";

      /**
       * Retrieve Mobile Device
       *
       * Create a request for the method "mobiledevices.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param resourceId Immutable id of Mobile Device
       * @since 1.13
       */
      protected Get(String customerId, String resourceId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.MobileDevice.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.resourceId = Preconditions.checkNotNull(resourceId, "Required parameter resourceId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Get setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Mobile Device */
      @com.google.api.client.util.Key
      private String resourceId;

      /** Immutable id of Mobile Device
       */
      public String getResourceId() {
        return resourceId;
      }

      /** Immutable id of Mobile Device */
      public Get setResourceId(String resourceId) {
        this.resourceId = resourceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Get setProjection(String projection) {
        this.projection = projection;
        return this;
      }

    }
    /**
     * Retrieve all Mobile Devices of a customer (paginated)
     *
     * Create a request for the method "mobiledevices.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @return the request
     */
    public List list(String customerId) throws java.io.IOException {
      List result = new List(customerId);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.MobileDevices> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile";

      /**
       * Retrieve all Mobile Devices of a customer (paginated)
       *
       * Create a request for the method "mobiledevices.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @since 1.13
       */
      protected List(String customerId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.MobileDevices.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public List setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Column to use for sorting results */
      @com.google.api.client.util.Key
      private String orderBy;

      /** Column to use for sorting results
       */
      public String getOrderBy() {
        return orderBy;
      }

      /** Column to use for sorting results */
      public List setOrderBy(String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public List setProjection(String projection) {
        this.projection = projection;
        return this;
      }

      /** Maximum number of results to return. Default is 100 */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Maximum number of results to return. Default is 100

     [minimum: 1]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 100 */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page in the list
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Whether to return results in ascending or descending order. Only of use when orderBy is also
     * used
     */
      @com.google.api.client.util.Key
      private String sortOrder;

      /** Whether to return results in ascending or descending order. Only of use when orderBy is also used
       */
      public String getSortOrder() {
        return sortOrder;
      }

      /**
     * Whether to return results in ascending or descending order. Only of use when orderBy is also
     * used
     */
      public List setSortOrder(String sortOrder) {
        this.sortOrder = sortOrder;
        return this;
      }

      /**
     * Search string in the format given at
     * http://support.google.com/a/bin/answer.py?hl=en=1408863#search
     */
      @com.google.api.client.util.Key
      private String query;

      /** Search string in the format given at http://support.google.com/a/bin/answer.py?hl=en=1408863#search
       */
      public String getQuery() {
        return query;
      }

      /**
     * Search string in the format given at
     * http://support.google.com/a/bin/answer.py?hl=en=1408863#search
     */
      public List setQuery(String query) {
        this.query = query;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Orgunits collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Orgunits.List request = admin.orgunits().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Orgunits orgunits() {
    return new Orgunits();
  }

  /**
   * The "orgunits" collection of methods.
   */
  public class Orgunits {

    /**
     * Remove Organization Unit
     *
     * Create a request for the method "orgunits.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @return the request
     */
    public Delete delete(String customerId, java.util.List<String> orgUnitPath) throws java.io.IOException {
      Delete result = new Delete(customerId, orgUnitPath);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Remove Organization Unit
       *
       * Create a request for the method "orgunits.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @since 1.13
       */
      protected Delete(String customerId, java.util.List<String> orgUnitPath) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Delete setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Delete setOrgUnitPath(java.util.List<String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

    }
    /**
     * Retrieve Organization Unit
     *
     * Create a request for the method "orgunits.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @return the request
     */
    public Get get(String customerId, java.util.List<String> orgUnitPath) throws java.io.IOException {
      Get result = new Get(customerId, orgUnitPath);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Retrieve Organization Unit
       *
       * Create a request for the method "orgunits.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @since 1.13
       */
      protected Get(String customerId, java.util.List<String> orgUnitPath) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Get setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Get setOrgUnitPath(java.util.List<String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

    }
    /**
     * Add Organization Unit
     *
     * Create a request for the method "orgunits.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
     * @return the request
     */
    public Insert insert(String customerId, com.google.api.services.admin.directory.model.OrgUnit content) throws java.io.IOException {
      Insert result = new Insert(customerId, content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits";

      /**
       * Add Organization Unit
       *
       * Create a request for the method "orgunits.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
       * @since 1.13
       */
      protected Insert(String customerId, com.google.api.services.admin.directory.model.OrgUnit content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Insert setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

    }
    /**
     * Retrieve all Organization Units
     *
     * Create a request for the method "orgunits.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @return the request
     */
    public List list(String customerId) throws java.io.IOException {
      List result = new List(customerId);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnits> {

      private static final String REST_PATH = "customer/{customerId}/orgunits";

      /**
       * Retrieve all Organization Units
       *
       * Create a request for the method "orgunits.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @since 1.13
       */
      protected List(String customerId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.OrgUnits.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public List setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Whether to return all sub-organizations or just immediate children */
      @com.google.api.client.util.Key
      private String type;

      /** Whether to return all sub-organizations or just immediate children
       */
      public String getType() {
        return type;
      }

      /** Whether to return all sub-organizations or just immediate children */
      public List setType(String type) {
        this.type = type;
        return this;
      }

      /** the URL-encoded organization unit */
      @com.google.api.client.util.Key
      private String orgUnitPath;

      /** the URL-encoded organization unit
       */
      public String getOrgUnitPath() {
        return orgUnitPath;
      }

      /** the URL-encoded organization unit */
      public List setOrgUnitPath(String orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

    }
    /**
     * Update Organization Unit. This method supports patch semantics.
     *
     * Create a request for the method "orgunits.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
     * @return the request
     */
    public Patch patch(String customerId, java.util.List<String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) throws java.io.IOException {
      Patch result = new Patch(customerId, orgUnitPath, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Update Organization Unit. This method supports patch semantics.
       *
       * Create a request for the method "orgunits.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
       * @since 1.13
       */
      protected Patch(String customerId, java.util.List<String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Patch setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Patch setOrgUnitPath(java.util.List<String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

    }
    /**
     * Update Organization Unit
     *
     * Create a request for the method "orgunits.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
     * @return the request
     */
    public Update update(String customerId, java.util.List<String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) throws java.io.IOException {
      Update result = new Update(customerId, orgUnitPath, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Update Organization Unit
       *
       * Create a request for the method "orgunits.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
       * @since 1.13
       */
      protected Update(String customerId, java.util.List<String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private String customerId;

      /** Immutable id of the Google Apps account
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Update setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Update setOrgUnitPath(java.util.List<String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Users collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Users.List request = admin.users().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Users users() {
    return new Users();
  }

  /**
   * The "users" collection of methods.
   */
  public class Users {

    /**
     * Delete user
     *
     * Create a request for the method "users.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user
     * @return the request
     */
    public Delete delete(String userKey) throws java.io.IOException {
      Delete result = new Delete(userKey);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * Delete user
       *
       * Create a request for the method "users.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user
       * @since 1.13
       */
      protected Delete(String userKey) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user */
      @com.google.api.client.util.Key
      private String userKey;

      /** Email or immutable Id of the user
       */
      public String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user */
      public Delete setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }
    /**
     * retrieve user
     *
     * Create a request for the method "users.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user
     * @return the request
     */
    public Get get(String userKey) throws java.io.IOException {
      Get result = new Get(userKey);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * retrieve user
       *
       * Create a request for the method "users.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user
       * @since 1.13
       */
      protected Get(String userKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.User.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user */
      @com.google.api.client.util.Key
      private String userKey;

      /** Email or immutable Id of the user
       */
      public String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user */
      public Get setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }
    /**
     * create user.
     *
     * Create a request for the method "users.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.admin.directory.model.User}
     * @return the request
     */
    public Insert insert(com.google.api.services.admin.directory.model.User content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users";

      /**
       * create user.
       *
       * Create a request for the method "users.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.admin.directory.model.User}
       * @since 1.13
       */
      protected Insert(com.google.api.services.admin.directory.model.User content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.User.class);
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

    }
    /**
     * Retrieve either deleted users or all users in a domain (paginated)
     *
     * Create a request for the method "users.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Users> {

      private static final String REST_PATH = "users";

      /**
       * Retrieve either deleted users or all users in a domain (paginated)
       *
       * Create a request for the method "users.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Users.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * Immutable id of the Google Apps account. In case of multi-domain, to fetch all users for a
     * customer, fill this field instead of domain.
     */
      @com.google.api.client.util.Key
      private String customer;

      /** Immutable id of the Google Apps account. In case of multi-domain, to fetch all users for a
     customer, fill this field instead of domain.
       */
      public String getCustomer() {
        return customer;
      }

      /**
     * Immutable id of the Google Apps account. In case of multi-domain, to fetch all users for a
     * customer, fill this field instead of domain.
     */
      public List setCustomer(String customer) {
        this.customer = customer;
        return this;
      }

      /** Column to use for sorting results */
      @com.google.api.client.util.Key
      private String orderBy;

      /** Column to use for sorting results
       */
      public String getOrderBy() {
        return orderBy;
      }

      /** Column to use for sorting results */
      public List setOrderBy(String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /**
     * Name of the domain. Fill this field to get users from only this domain. To return all users
     * in a multi-domain fill customer field instead.
     */
      @com.google.api.client.util.Key
      private String domain;

      /** Name of the domain. Fill this field to get users from only this domain. To return all users in a
     multi-domain fill customer field instead.
       */
      public String getDomain() {
        return domain;
      }

      /**
     * Name of the domain. Fill this field to get users from only this domain. To return all users
     * in a multi-domain fill customer field instead.
     */
      public List setDomain(String domain) {
        this.domain = domain;
        return this;
      }

      /** If set to true retrieves the list of deleted users. Default is false */
      @com.google.api.client.util.Key
      private String showDeleted;

      /** If set to true retrieves the list of deleted users. Default is false
       */
      public String getShowDeleted() {
        return showDeleted;
      }

      /** If set to true retrieves the list of deleted users. Default is false */
      public List setShowDeleted(String showDeleted) {
        this.showDeleted = showDeleted;
        return this;
      }

      /** Maximum number of results to return. Default is 100. Max allowed is 500 */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Maximum number of results to return. Default is 100. Max allowed is 500

     [minimum: 1] [maximum: 500]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 100. Max allowed is 500 */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page in the list
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Whether to return results in ascending or descending order. */
      @com.google.api.client.util.Key
      private String sortOrder;

      /** Whether to return results in ascending or descending order.
       */
      public String getSortOrder() {
        return sortOrder;
      }

      /** Whether to return results in ascending or descending order. */
      public List setSortOrder(String sortOrder) {
        this.sortOrder = sortOrder;
        return this;
      }

      /**
     * Query string for prefix matching searches. Should be of the form "key:value" where key can be
     * "email", "firstName" or "lastName".
     */
      @com.google.api.client.util.Key
      private String query;

      /** Query string for prefix matching searches. Should be of the form "key:value" where key can be
     "email", "firstName" or "lastName".
       */
      public String getQuery() {
        return query;
      }

      /**
     * Query string for prefix matching searches. Should be of the form "key:value" where key can be
     * "email", "firstName" or "lastName".
     */
      public List setQuery(String query) {
        this.query = query;
        return this;
      }

    }
    /**
     * change admin status of a user
     *
     * Create a request for the method "users.makeAdmin".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link MakeAdmin#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user as admin
     * @param content the {@link com.google.api.services.admin.directory.model.UserMakeAdmin}
     * @return the request
     */
    public MakeAdmin makeAdmin(String userKey, com.google.api.services.admin.directory.model.UserMakeAdmin content) throws java.io.IOException {
      MakeAdmin result = new MakeAdmin(userKey, content);
      initialize(result);
      return result;
    }

    public class MakeAdmin extends DirectoryRequest<Void> {

      private static final String REST_PATH = "users/{userKey}/makeAdmin";

      /**
       * change admin status of a user
       *
       * Create a request for the method "users.makeAdmin".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link MakeAdmin#execute()} method to invoke the remote operation. <p>
       * {@link MakeAdmin#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user as admin
       * @param content the {@link com.google.api.services.admin.directory.model.UserMakeAdmin}
       * @since 1.13
       */
      protected MakeAdmin(String userKey, com.google.api.services.admin.directory.model.UserMakeAdmin content) {
        super(Directory.this, "POST", REST_PATH, content, Void.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public MakeAdmin setAlt(String alt) {
        return (MakeAdmin) super.setAlt(alt);
      }

      @Override
      public MakeAdmin setFields(String fields) {
        return (MakeAdmin) super.setFields(fields);
      }

      @Override
      public MakeAdmin setKey(String key) {
        return (MakeAdmin) super.setKey(key);
      }

      @Override
      public MakeAdmin setOauthToken(String oauthToken) {
        return (MakeAdmin) super.setOauthToken(oauthToken);
      }

      @Override
      public MakeAdmin setPrettyPrint(Boolean prettyPrint) {
        return (MakeAdmin) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public MakeAdmin setQuotaUser(String quotaUser) {
        return (MakeAdmin) super.setQuotaUser(quotaUser);
      }

      @Override
      public MakeAdmin setUserIp(String userIp) {
        return (MakeAdmin) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user as admin */
      @com.google.api.client.util.Key
      private String userKey;

      /** Email or immutable Id of the user as admin
       */
      public String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user as admin */
      public MakeAdmin setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }
    /**
     * update user. This method supports patch semantics.
     *
     * Create a request for the method "users.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
     * @param content the {@link com.google.api.services.admin.directory.model.User}
     * @return the request
     */
    public Patch patch(String userKey, com.google.api.services.admin.directory.model.User content) throws java.io.IOException {
      Patch result = new Patch(userKey, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * update user. This method supports patch semantics.
       *
       * Create a request for the method "users.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
       * @param content the {@link com.google.api.services.admin.directory.model.User}
       * @since 1.13
       */
      protected Patch(String userKey, com.google.api.services.admin.directory.model.User content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.User.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      @com.google.api.client.util.Key
      private String userKey;

      /** Email or immutable Id of the user. If Id, it should match with id of user object
       */
      public String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      public Patch setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }
    /**
     * Undelete a deleted user
     *
     * Create a request for the method "users.undelete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Undelete#execute()} method to invoke the remote operation.
     *
     * @param userKey The immutable id of the user
     * @return the request
     */
    public Undelete undelete(String userKey) throws java.io.IOException {
      Undelete result = new Undelete(userKey);
      initialize(result);
      return result;
    }

    public class Undelete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "users/{userKey}/undelete";

      /**
       * Undelete a deleted user
       *
       * Create a request for the method "users.undelete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Undelete#execute()} method to invoke the remote operation. <p>
       * {@link Undelete#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param userKey The immutable id of the user
       * @since 1.13
       */
      protected Undelete(String userKey) {
        super(Directory.this, "POST", REST_PATH, null, Void.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Undelete setAlt(String alt) {
        return (Undelete) super.setAlt(alt);
      }

      @Override
      public Undelete setFields(String fields) {
        return (Undelete) super.setFields(fields);
      }

      @Override
      public Undelete setKey(String key) {
        return (Undelete) super.setKey(key);
      }

      @Override
      public Undelete setOauthToken(String oauthToken) {
        return (Undelete) super.setOauthToken(oauthToken);
      }

      @Override
      public Undelete setPrettyPrint(Boolean prettyPrint) {
        return (Undelete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Undelete setQuotaUser(String quotaUser) {
        return (Undelete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Undelete setUserIp(String userIp) {
        return (Undelete) super.setUserIp(userIp);
      }

      /** The immutable id of the user */
      @com.google.api.client.util.Key
      private String userKey;

      /** The immutable id of the user
       */
      public String getUserKey() {
        return userKey;
      }

      /** The immutable id of the user */
      public Undelete setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }
    /**
     * update user
     *
     * Create a request for the method "users.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
     * @param content the {@link com.google.api.services.admin.directory.model.User}
     * @return the request
     */
    public Update update(String userKey, com.google.api.services.admin.directory.model.User content) throws java.io.IOException {
      Update result = new Update(userKey, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * update user
       *
       * Create a request for the method "users.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
       * @param content the {@link com.google.api.services.admin.directory.model.User}
       * @since 1.13
       */
      protected Update(String userKey, com.google.api.services.admin.directory.model.User content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.User.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      @com.google.api.client.util.Key
      private String userKey;

      /** Email or immutable Id of the user. If Id, it should match with id of user object
       */
      public String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      public Update setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

    }

    /**
     * An accessor for creating requests from the Aliases collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Directory admin = new Directory(...);}
     *   {@code Directory.Aliases.List request = admin.aliases().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Aliases aliases() {
      return new Aliases();
    }

    /**
     * The "aliases" collection of methods.
     */
    public class Aliases {

      /**
       * Remove a alias for the user
       *
       * Create a request for the method "aliases.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param alias The alias to be removed
       * @return the request
       */
      public Delete delete(String userKey, String alias) throws java.io.IOException {
        Delete result = new Delete(userKey, alias);
        initialize(result);
        return result;
      }

      public class Delete extends DirectoryRequest<Void> {

        private static final String REST_PATH = "users/{userKey}/aliases/{alias}";

        /**
         * Remove a alias for the user
         *
         * Create a request for the method "aliases.delete".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param alias The alias to be removed
         * @since 1.13
         */
        protected Delete(String userKey, String alias) {
          super(Directory.this, "DELETE", REST_PATH, null, Void.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
          this.alias = Preconditions.checkNotNull(alias, "Required parameter alias must be specified.");
        }

        @Override
        public Delete setAlt(String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Delete setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

        /** The alias to be removed */
        @com.google.api.client.util.Key
        private String alias;

        /** The alias to be removed
         */
        public String getAlias() {
          return alias;
        }

        /** The alias to be removed */
        public Delete setAlias(String alias) {
          this.alias = alias;
          return this;
        }

      }
      /**
       * Add a alias for the user
       *
       * Create a request for the method "aliases.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.Alias}
       * @return the request
       */
      public Insert insert(String userKey, com.google.api.services.admin.directory.model.Alias content) throws java.io.IOException {
        Insert result = new Insert(userKey, content);
        initialize(result);
        return result;
      }

      public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Alias> {

        private static final String REST_PATH = "users/{userKey}/aliases";

        /**
         * Add a alias for the user
         *
         * Create a request for the method "aliases.insert".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
         * Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param content the {@link com.google.api.services.admin.directory.model.Alias}
         * @since 1.13
         */
        protected Insert(String userKey, com.google.api.services.admin.directory.model.Alias content) {
          super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Alias.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Insert setAlt(String alt) {
          return (Insert) super.setAlt(alt);
        }

        @Override
        public Insert setFields(String fields) {
          return (Insert) super.setFields(fields);
        }

        @Override
        public Insert setKey(String key) {
          return (Insert) super.setKey(key);
        }

        @Override
        public Insert setOauthToken(String oauthToken) {
          return (Insert) super.setOauthToken(oauthToken);
        }

        @Override
        public Insert setPrettyPrint(Boolean prettyPrint) {
          return (Insert) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Insert setQuotaUser(String quotaUser) {
          return (Insert) super.setQuotaUser(quotaUser);
        }

        @Override
        public Insert setUserIp(String userIp) {
          return (Insert) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Insert setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

      }
      /**
       * List all aliases for a user
       *
       * Create a request for the method "aliases.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @return the request
       */
      public List list(String userKey) throws java.io.IOException {
        List result = new List(userKey);
        initialize(result);
        return result;
      }

      public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Aliases> {

        private static final String REST_PATH = "users/{userKey}/aliases";

        /**
         * List all aliases for a user
         *
         * Create a request for the method "aliases.list".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
         * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @since 1.13
         */
        protected List(String userKey) {
          super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Aliases.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public List setAlt(String alt) {
          return (List) super.setAlt(alt);
        }

        @Override
        public List setFields(String fields) {
          return (List) super.setFields(fields);
        }

        @Override
        public List setKey(String key) {
          return (List) super.setKey(key);
        }

        @Override
        public List setOauthToken(String oauthToken) {
          return (List) super.setOauthToken(oauthToken);
        }

        @Override
        public List setPrettyPrint(Boolean prettyPrint) {
          return (List) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public List setQuotaUser(String quotaUser) {
          return (List) super.setQuotaUser(quotaUser);
        }

        @Override
        public List setUserIp(String userIp) {
          return (List) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public List setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

      }

    }
    /**
     * An accessor for creating requests from the Photos collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Directory admin = new Directory(...);}
     *   {@code Directory.Photos.List request = admin.photos().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Photos photos() {
      return new Photos();
    }

    /**
     * The "photos" collection of methods.
     */
    public class Photos {

      /**
       * Remove photos for the user
       *
       * Create a request for the method "photos.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @return the request
       */
      public Delete delete(String userKey) throws java.io.IOException {
        Delete result = new Delete(userKey);
        initialize(result);
        return result;
      }

      public class Delete extends DirectoryRequest<Void> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Remove photos for the user
         *
         * Create a request for the method "photos.delete".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @since 1.13
         */
        protected Delete(String userKey) {
          super(Directory.this, "DELETE", REST_PATH, null, Void.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Delete setAlt(String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Delete setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

      }
      /**
       * Retrieve photo of a user
       *
       * Create a request for the method "photos.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @return the request
       */
      public Get get(String userKey) throws java.io.IOException {
        Get result = new Get(userKey);
        initialize(result);
        return result;
      }

      public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.UserPhoto> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Retrieve photo of a user
         *
         * Create a request for the method "photos.get".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
         * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @since 1.13
         */
        protected Get(String userKey) {
          super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.UserPhoto.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public Get setAlt(String alt) {
          return (Get) super.setAlt(alt);
        }

        @Override
        public Get setFields(String fields) {
          return (Get) super.setFields(fields);
        }

        @Override
        public Get setKey(String key) {
          return (Get) super.setKey(key);
        }

        @Override
        public Get setOauthToken(String oauthToken) {
          return (Get) super.setOauthToken(oauthToken);
        }

        @Override
        public Get setPrettyPrint(Boolean prettyPrint) {
          return (Get) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Get setQuotaUser(String quotaUser) {
          return (Get) super.setQuotaUser(quotaUser);
        }

        @Override
        public Get setUserIp(String userIp) {
          return (Get) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Get setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

      }
      /**
       * Add a photo for the user. This method supports patch semantics.
       *
       * Create a request for the method "photos.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
       * @return the request
       */
      public Patch patch(String userKey, com.google.api.services.admin.directory.model.UserPhoto content) throws java.io.IOException {
        Patch result = new Patch(userKey, content);
        initialize(result);
        return result;
      }

      public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.UserPhoto> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Add a photo for the user. This method supports patch semantics.
         *
         * Create a request for the method "photos.patch".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
         * Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
         * @since 1.13
         */
        protected Patch(String userKey, com.google.api.services.admin.directory.model.UserPhoto content) {
          super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.UserPhoto.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Patch setAlt(String alt) {
          return (Patch) super.setAlt(alt);
        }

        @Override
        public Patch setFields(String fields) {
          return (Patch) super.setFields(fields);
        }

        @Override
        public Patch setKey(String key) {
          return (Patch) super.setKey(key);
        }

        @Override
        public Patch setOauthToken(String oauthToken) {
          return (Patch) super.setOauthToken(oauthToken);
        }

        @Override
        public Patch setPrettyPrint(Boolean prettyPrint) {
          return (Patch) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Patch setQuotaUser(String quotaUser) {
          return (Patch) super.setQuotaUser(quotaUser);
        }

        @Override
        public Patch setUserIp(String userIp) {
          return (Patch) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Patch setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

      }
      /**
       * Add a photo for the user
       *
       * Create a request for the method "photos.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
       * @return the request
       */
      public Update update(String userKey, com.google.api.services.admin.directory.model.UserPhoto content) throws java.io.IOException {
        Update result = new Update(userKey, content);
        initialize(result);
        return result;
      }

      public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.UserPhoto> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Add a photo for the user
         *
         * Create a request for the method "photos.update".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
         * Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
         * immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
         * @since 1.13
         */
        protected Update(String userKey, com.google.api.services.admin.directory.model.UserPhoto content) {
          super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.UserPhoto.class);
          this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Update setAlt(String alt) {
          return (Update) super.setAlt(alt);
        }

        @Override
        public Update setFields(String fields) {
          return (Update) super.setFields(fields);
        }

        @Override
        public Update setKey(String key) {
          return (Update) super.setKey(key);
        }

        @Override
        public Update setOauthToken(String oauthToken) {
          return (Update) super.setOauthToken(oauthToken);
        }

        @Override
        public Update setPrettyPrint(Boolean prettyPrint) {
          return (Update) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Update setQuotaUser(String quotaUser) {
          return (Update) super.setQuotaUser(quotaUser);
        }

        @Override
        public Update setUserIp(String userIp) {
          return (Update) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private String userKey;

        /** Email or immutable Id of the user
         */
        public String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Update setUserKey(String userKey) {
          this.userKey = userKey;
          return this;
        }

      }

    }
  }

  /**
   * Builder for {@link Directory}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Directory}. */
    @Override
    public Directory build() {
      return new Directory(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link DirectoryRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setDirectoryRequestInitializer(
        DirectoryRequestInitializer directoryRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(directoryRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
