/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.admin.directory.model;

import com.google.api.client.json.GenericJson;
import com.google.api.client.util.DateTime;

/**
 * JSON template for Chrome Os Device resource in Apps Directory API.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the Google Apps Directory API. For a detailed explanation
 * see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class ChromeOsDevice extends GenericJson {

  /**
   * Address or location of the device as noted by the administrator
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String annotatedLocation;

  /**
   * User of the device
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String annotatedUser;

  /**
   * Chromebook boot mode (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String bootMode;

  /**
   * Unique identifier of Chrome OS Device (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String deviceId;

  /**
   * Chromebook firmware version (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String firmwareVersion;

  /**
   * Kind of resource this is.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String kind;

  /**
   * Date and time the device was last enrolled (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime lastEnrollmentTime;

  /**
   * Date and time the device was last synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime lastSync;

  /**
   * Chromebook Mac Address (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String macAddress;

  /**
   * Mobile Equipment identifier for the 3G mobile card in the Chromebook (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String meid;

  /**
   * Chromebook Model (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String model;

  /**
   * Notes added by the administrator
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String notes;

  /**
   * Chromebook order number (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String orderNumber;

  /**
   * OrgUnit of the device (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String orgUnitPath;

  /**
   * Chromebook Os Version (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String osVersion;

  /**
   * Chromebook platform version (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String platformVersion;

  /**
   * Chromebook serial number (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String serialNumber;

  /**
   * status of the device (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String status;

  /**
   * Final date the device will be supported (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime supportEndDate;

  /**
   * Will Chromebook auto reniew after support end date (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean willAutoRenew;

  /**
   * Address or location of the device as noted by the administrator
   * The value returned may be {@code null}.
   */
  public String getAnnotatedLocation() {
    return annotatedLocation;
  }

  /**
   * Address or location of the device as noted by the administrator
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setAnnotatedLocation(String annotatedLocation) {
    this.annotatedLocation = annotatedLocation;
    return this;
  }

  /**
   * User of the device
   * The value returned may be {@code null}.
   */
  public String getAnnotatedUser() {
    return annotatedUser;
  }

  /**
   * User of the device
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setAnnotatedUser(String annotatedUser) {
    this.annotatedUser = annotatedUser;
    return this;
  }

  /**
   * Chromebook boot mode (Read-only)
   * The value returned may be {@code null}.
   */
  public String getBootMode() {
    return bootMode;
  }

  /**
   * Chromebook boot mode (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setBootMode(String bootMode) {
    this.bootMode = bootMode;
    return this;
  }

  /**
   * Unique identifier of Chrome OS Device (Read-only)
   * The value returned may be {@code null}.
   */
  public String getDeviceId() {
    return deviceId;
  }

  /**
   * Unique identifier of Chrome OS Device (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setDeviceId(String deviceId) {
    this.deviceId = deviceId;
    return this;
  }

  /**
   * Chromebook firmware version (Read-only)
   * The value returned may be {@code null}.
   */
  public String getFirmwareVersion() {
    return firmwareVersion;
  }

  /**
   * Chromebook firmware version (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setFirmwareVersion(String firmwareVersion) {
    this.firmwareVersion = firmwareVersion;
    return this;
  }

  /**
   * Kind of resource this is.
   * The value returned may be {@code null}.
   */
  public String getKind() {
    return kind;
  }

  /**
   * Kind of resource this is.
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setKind(String kind) {
    this.kind = kind;
    return this;
  }

  /**
   * Date and time the device was last enrolled (Read-only)
   * The value returned may be {@code null}.
   */
  public DateTime getLastEnrollmentTime() {
    return lastEnrollmentTime;
  }

  /**
   * Date and time the device was last enrolled (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setLastEnrollmentTime(DateTime lastEnrollmentTime) {
    this.lastEnrollmentTime = lastEnrollmentTime;
    return this;
  }

  /**
   * Date and time the device was last synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value returned may be {@code null}.
   */
  public DateTime getLastSync() {
    return lastSync;
  }

  /**
   * Date and time the device was last synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setLastSync(DateTime lastSync) {
    this.lastSync = lastSync;
    return this;
  }

  /**
   * Chromebook Mac Address (Read-only)
   * The value returned may be {@code null}.
   */
  public String getMacAddress() {
    return macAddress;
  }

  /**
   * Chromebook Mac Address (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setMacAddress(String macAddress) {
    this.macAddress = macAddress;
    return this;
  }

  /**
   * Mobile Equipment identifier for the 3G mobile card in the Chromebook (Read-only)
   * The value returned may be {@code null}.
   */
  public String getMeid() {
    return meid;
  }

  /**
   * Mobile Equipment identifier for the 3G mobile card in the Chromebook (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setMeid(String meid) {
    this.meid = meid;
    return this;
  }

  /**
   * Chromebook Model (Read-only)
   * The value returned may be {@code null}.
   */
  public String getModel() {
    return model;
  }

  /**
   * Chromebook Model (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setModel(String model) {
    this.model = model;
    return this;
  }

  /**
   * Notes added by the administrator
   * The value returned may be {@code null}.
   */
  public String getNotes() {
    return notes;
  }

  /**
   * Notes added by the administrator
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setNotes(String notes) {
    this.notes = notes;
    return this;
  }

  /**
   * Chromebook order number (Read-only)
   * The value returned may be {@code null}.
   */
  public String getOrderNumber() {
    return orderNumber;
  }

  /**
   * Chromebook order number (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setOrderNumber(String orderNumber) {
    this.orderNumber = orderNumber;
    return this;
  }

  /**
   * OrgUnit of the device (Read-only)
   * The value returned may be {@code null}.
   */
  public String getOrgUnitPath() {
    return orgUnitPath;
  }

  /**
   * OrgUnit of the device (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setOrgUnitPath(String orgUnitPath) {
    this.orgUnitPath = orgUnitPath;
    return this;
  }

  /**
   * Chromebook Os Version (Read-only)
   * The value returned may be {@code null}.
   */
  public String getOsVersion() {
    return osVersion;
  }

  /**
   * Chromebook Os Version (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setOsVersion(String osVersion) {
    this.osVersion = osVersion;
    return this;
  }

  /**
   * Chromebook platform version (Read-only)
   * The value returned may be {@code null}.
   */
  public String getPlatformVersion() {
    return platformVersion;
  }

  /**
   * Chromebook platform version (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setPlatformVersion(String platformVersion) {
    this.platformVersion = platformVersion;
    return this;
  }

  /**
   * Chromebook serial number (Read-only)
   * The value returned may be {@code null}.
   */
  public String getSerialNumber() {
    return serialNumber;
  }

  /**
   * Chromebook serial number (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setSerialNumber(String serialNumber) {
    this.serialNumber = serialNumber;
    return this;
  }

  /**
   * status of the device (Read-only)
   * The value returned may be {@code null}.
   */
  public String getStatus() {
    return status;
  }

  /**
   * status of the device (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setStatus(String status) {
    this.status = status;
    return this;
  }

  /**
   * Final date the device will be supported (Read-only)
   * The value returned may be {@code null}.
   */
  public DateTime getSupportEndDate() {
    return supportEndDate;
  }

  /**
   * Final date the device will be supported (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setSupportEndDate(DateTime supportEndDate) {
    this.supportEndDate = supportEndDate;
    return this;
  }

  /**
   * Will Chromebook auto reniew after support end date (Read-only)
   * The value returned may be {@code null}.
   */
  public Boolean getWillAutoRenew() {
    return willAutoRenew;
  }

  /**
   * Will Chromebook auto reniew after support end date (Read-only)
   * The value set may be {@code null}.
   */
  public ChromeOsDevice setWillAutoRenew(Boolean willAutoRenew) {
    this.willAutoRenew = willAutoRenew;
    return this;
  }

}
