/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.admin.directory.model;

import com.google.api.client.json.GenericJson;
import com.google.api.client.util.DateTime;

/**
 * JSON template for Mobile Device resource in Apps Directory API.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the Google Apps Directory API. For a detailed explanation
 * see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class MobileDevice extends GenericJson {

  /**
   * List of applications installed on Mobile Device
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Applications> applications;

  static {
    // hack to force ProGuard to consider Applications used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Applications.class);
  }

  /**
   * Mobile Device serial number (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String deviceId;

  /**
   * List of owner user's email addresses (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> email;

  /**
   * Date and time the device was first synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime firstSync;

  /**
   * Mobile Device Hardware Id (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String hardwareId;

  /**
   * Kind of resource this is.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String kind;

  /**
   * Date and time the device was last synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime lastSync;

  /**
   * Name of the model of the device
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String model;

  /**
   * List of owner user's names (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> name;

  /**
   * Name of the mobile operating system
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String os;

  /**
   * Unique identifier of Mobile Device (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String resourceId;

  /**
   * Status of the device (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String status;

  /**
   * The type of device (Read-only)
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String type;

  /**
   * Mobile Device user agent
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String userAgent;

  /**
   * List of applications installed on Mobile Device
   * The value returned may be {@code null}.
   */
  public java.util.List<Applications> getApplications() {
    return applications;
  }

  /**
   * List of applications installed on Mobile Device
   * The value set may be {@code null}.
   */
  public MobileDevice setApplications(java.util.List<Applications> applications) {
    this.applications = applications;
    return this;
  }

  /**
   * Mobile Device serial number (Read-only)
   * The value returned may be {@code null}.
   */
  public String getDeviceId() {
    return deviceId;
  }

  /**
   * Mobile Device serial number (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setDeviceId(String deviceId) {
    this.deviceId = deviceId;
    return this;
  }

  /**
   * List of owner user's email addresses (Read-only)
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getEmail() {
    return email;
  }

  /**
   * List of owner user's email addresses (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setEmail(java.util.List<String> email) {
    this.email = email;
    return this;
  }

  /**
   * Date and time the device was first synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value returned may be {@code null}.
   */
  public DateTime getFirstSync() {
    return firstSync;
  }

  /**
   * Date and time the device was first synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setFirstSync(DateTime firstSync) {
    this.firstSync = firstSync;
    return this;
  }

  /**
   * Mobile Device Hardware Id (Read-only)
   * The value returned may be {@code null}.
   */
  public String getHardwareId() {
    return hardwareId;
  }

  /**
   * Mobile Device Hardware Id (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setHardwareId(String hardwareId) {
    this.hardwareId = hardwareId;
    return this;
  }

  /**
   * Kind of resource this is.
   * The value returned may be {@code null}.
   */
  public String getKind() {
    return kind;
  }

  /**
   * Kind of resource this is.
   * The value set may be {@code null}.
   */
  public MobileDevice setKind(String kind) {
    this.kind = kind;
    return this;
  }

  /**
   * Date and time the device was last synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value returned may be {@code null}.
   */
  public DateTime getLastSync() {
    return lastSync;
  }

  /**
   * Date and time the device was last synchronized with the policy settings in the Google Apps
   * administrator control panel (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setLastSync(DateTime lastSync) {
    this.lastSync = lastSync;
    return this;
  }

  /**
   * Name of the model of the device
   * The value returned may be {@code null}.
   */
  public String getModel() {
    return model;
  }

  /**
   * Name of the model of the device
   * The value set may be {@code null}.
   */
  public MobileDevice setModel(String model) {
    this.model = model;
    return this;
  }

  /**
   * List of owner user's names (Read-only)
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getName() {
    return name;
  }

  /**
   * List of owner user's names (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setName(java.util.List<String> name) {
    this.name = name;
    return this;
  }

  /**
   * Name of the mobile operating system
   * The value returned may be {@code null}.
   */
  public String getOs() {
    return os;
  }

  /**
   * Name of the mobile operating system
   * The value set may be {@code null}.
   */
  public MobileDevice setOs(String os) {
    this.os = os;
    return this;
  }

  /**
   * Unique identifier of Mobile Device (Read-only)
   * The value returned may be {@code null}.
   */
  public String getResourceId() {
    return resourceId;
  }

  /**
   * Unique identifier of Mobile Device (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setResourceId(String resourceId) {
    this.resourceId = resourceId;
    return this;
  }

  /**
   * Status of the device (Read-only)
   * The value returned may be {@code null}.
   */
  public String getStatus() {
    return status;
  }

  /**
   * Status of the device (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setStatus(String status) {
    this.status = status;
    return this;
  }

  /**
   * The type of device (Read-only)
   * The value returned may be {@code null}.
   */
  public String getType() {
    return type;
  }

  /**
   * The type of device (Read-only)
   * The value set may be {@code null}.
   */
  public MobileDevice setType(String type) {
    this.type = type;
    return this;
  }

  /**
   * Mobile Device user agent
   * The value returned may be {@code null}.
   */
  public String getUserAgent() {
    return userAgent;
  }

  /**
   * Mobile Device user agent
   * The value set may be {@code null}.
   */
  public MobileDevice setUserAgent(String userAgent) {
    this.userAgent = userAgent;
    return this;
  }

  /**
   * Model definition for MobileDeviceApplications.
   */
  public static final class Applications extends GenericJson {

    /**
     * Display name of application
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String displayName;

    /**
     * Package name of application
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String packageName;

    /**
     * List of Permissions for application
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> permission;

    /**
     * Version code of application
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer versionCode;

    /**
     * Version name of application
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String versionName;

    /**
     * Display name of application
     * The value returned may be {@code null}.
     */
    public String getDisplayName() {
      return displayName;
    }

    /**
     * Display name of application
     * The value set may be {@code null}.
     */
    public Applications setDisplayName(String displayName) {
      this.displayName = displayName;
      return this;
    }

    /**
     * Package name of application
     * The value returned may be {@code null}.
     */
    public String getPackageName() {
      return packageName;
    }

    /**
     * Package name of application
     * The value set may be {@code null}.
     */
    public Applications setPackageName(String packageName) {
      this.packageName = packageName;
      return this;
    }

    /**
     * List of Permissions for application
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getPermission() {
      return permission;
    }

    /**
     * List of Permissions for application
     * The value set may be {@code null}.
     */
    public Applications setPermission(java.util.List<String> permission) {
      this.permission = permission;
      return this;
    }

    /**
     * Version code of application
     * The value returned may be {@code null}.
     */
    public Integer getVersionCode() {
      return versionCode;
    }

    /**
     * Version code of application
     * The value set may be {@code null}.
     */
    public Applications setVersionCode(Integer versionCode) {
      this.versionCode = versionCode;
      return this;
    }

    /**
     * Version name of application
     * The value returned may be {@code null}.
     */
    public String getVersionName() {
      return versionName;
    }

    /**
     * Version name of application
     * The value set may be {@code null}.
     */
    public Applications setVersionName(String versionName) {
      this.versionName = versionName;
      return this;
    }

  }

}
